package com.swing.camera.swing_camera

import android.annotation.SuppressLint
import android.app.Activity
import android.util.Log
import android.util.Size
import android.view.Surface
import androidx.camera.core.Camera
import androidx.camera.core.CameraSelector
import androidx.camera.core.ImageAnalysis
import androidx.camera.core.Preview
import androidx.camera.lifecycle.ProcessCameraProvider
import androidx.core.content.ContextCompat
import androidx.lifecycle.LifecycleOwner
import io.flutter.plugin.common.MethodChannel
import io.flutter.view.TextureRegistry

interface ICameraXPlugin {
    fun initCamera(result: MethodChannel.Result)

    fun startCamera()

    fun stopCamera()

    fun takePicture()

    fun switchCamera()

    fun setFlashMode()

    fun setZoomLevel()

    fun setFocusMode()
}

class CameraXPlugin constructor(
    private val activity: Activity,
    private val textureRegistry: TextureRegistry,
) : ICameraXPlugin {
    private var cameraX: Camera? = null
    private var cameraProcessProvider: ProcessCameraProvider? = null
    private var textureEntry: TextureRegistry.SurfaceTextureEntry? = null
    private var imageAnalyzer: ImageAnalysis.Analyzer? = null

    @SuppressLint("RestrictedApi")
    override fun initCamera(result: MethodChannel.Result) {
        val cameraProviderFuture = ProcessCameraProvider.getInstance(activity)
        val cameraExecutor = ContextCompat.getMainExecutor(activity)
        cameraProviderFuture.addListener({
            cameraProcessProvider = cameraProviderFuture.get()
            textureEntry = textureRegistry.createSurfaceTexture()
            val textureId = textureEntry?.id()
            val surfaceHolder = Preview.SurfaceProvider {
                val resolution = it.resolution
                val surfaceTexture = textureEntry?.surfaceTexture()
                surfaceTexture?.setDefaultBufferSize(resolution.width, resolution.height)
                val surface = surfaceTexture?.let { it1 -> Surface(it1) }
                if (surface != null) {
                    it.provideSurface(surface, cameraExecutor) {}
                }
            }
            val preview = Preview.Builder()
                .build()
                .apply {
                    setSurfaceProvider(surfaceHolder)
                }

            val lifecycleOwner = activity as LifecycleOwner

            // Analyzer
            val analysis = ImageAnalysis.Builder()
                .setBackpressureStrategy(ImageAnalysis.STRATEGY_KEEP_ONLY_LATEST)
                .setTargetResolution(
                    Size(
                        1280,
                        720
                    )
                ).build().apply {
                    imageAnalyzer?.let { setAnalyzer(cameraExecutor, it) }
                }

            try {
                cameraProcessProvider?.unbindAll()
                cameraX = cameraProcessProvider?.bindToLifecycle(
                    lifecycleOwner,
                    CameraSelector.DEFAULT_BACK_CAMERA,
                    preview,
                    analysis
                )
            } catch (e: Exception) {
                e.printStackTrace()
            }

            val resolution = preview.attachedSurfaceResolution ?: Size(1280, 720)
            val width = resolution.width.toDouble()
            val height = resolution.height.toDouble()
            Log.d("CameraXPlugin", "textureId: $textureId width: $width, height: $height")
            val updatedCameraParams = CameraParams(
                cameraId = textureId.toString(),
                previewWidth = width,
                previewHeight = height,
            )

            Log.d("CameraXPlugin", "updatedCameraParams: $updatedCameraParams")
            result.success(updatedCameraParams.toMap())
        }, cameraExecutor)
    }

    override fun startCamera() {

    }

    override fun stopCamera() {

    }

    override fun takePicture() {
    }

    override fun switchCamera() {
    }

    override fun setFlashMode() {
    }

    override fun setZoomLevel() {
    }

    override fun setFocusMode() {
    }

}
