package com.swing.camera.swing_camera

import android.app.Activity
import android.util.Log
import io.flutter.plugin.common.BinaryMessenger
import io.flutter.plugin.common.EventChannel
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.view.TextureRegistry

class MethodHandlerImpl constructor(
    messenger: BinaryMessenger,
    private val activity: Activity,
    private val textureRegistry: TextureRegistry,
) : MethodChannel.MethodCallHandler, EventChannel.StreamHandler {

    private var channel: MethodChannel? = null
    private var cameraXPlugin: CameraXPlugin? = null
    private var eventChannel: EventChannel.EventSink? = null

    init {
        channel = MethodChannel(messenger, Constants.CHANNEL_NAME)
        channel?.setMethodCallHandler(this)
        val eventChannel = EventChannel(messenger, Constants.EVENT_CHANNEL_NAME)
        eventChannel.setStreamHandler(this)

        cameraXPlugin = CameraXPlugin(activity, textureRegistry)
    }

    override fun onMethodCall(call: MethodCall, result: MethodChannel.Result) {
        when (call.method) {
            "getPlatformVersion" -> result.success("Android ${android.os.Build.VERSION.RELEASE}")
            "initCamera" -> {
                cameraXPlugin?.initCamera(result = result)
            }
            "startCamera" -> {
                cameraXPlugin?.startCamera()
            }
            "stopCamera" -> {
                cameraXPlugin?.stopCamera()
            }
            "takePicture" -> {
                cameraXPlugin?.takePicture()
            }
            "switchCamera" -> {
                cameraXPlugin?.switchCamera()
            }
            "setFlashMode" -> {
                cameraXPlugin?.setFlashMode()
            }
            "setZoomLevel" -> {
                cameraXPlugin?.setZoomLevel()
            }
            "setFocusMode" -> {
                cameraXPlugin?.setFocusMode()
            }
            else -> result.notImplemented()
        }
    }

    fun dispose() {
        channel?.setMethodCallHandler(null)
    }

    override fun onListen(arguments: Any?, events: EventChannel.EventSink?) {
        eventChannel = events
    }

    override fun onCancel(arguments: Any?) {

    }
}
