import 'package:flutter/cupertino.dart';
import 'package:flutter/services.dart';

import 'camera_params.dart';

mixin ISwingCameraController {
  Future<void> captureImage();

  Future<void> startCamera();

  Future<void> stopCamera();

  ValueNotifier<CameraParams?> get cameraParams;
}

class SwingCameraController
    implements ISwingCameraController {
  static const MethodChannel _channel =
      MethodChannel('swing_camera');
  // static const EventChannel _eventChannel =
  //     EventChannel('swing_camera_event');
  CameraParams? _cameraParams;

  @override
  Future<void> captureImage() async {}

  @override
  Future<void> startCamera() async {
    final camera = await _channel
        .invokeMapMethod<String, dynamic>('initCamera');

    if (camera == null) {
      throw Exception('Camera is null');
    }

    debugPrint('Camera: $camera');

    _cameraParams = CameraParams(
      cameraId: camera['cameraId'],
      previewSize: Size(
          camera['previewWidth'], camera['previewHeight']),
    );
  }

  @override
  Future<void> stopCamera() async {
    await _channel.invokeMethod('stopCamera');
  }

  @override
  ValueNotifier<CameraParams?> get cameraParams {
    return ValueNotifier(_cameraParams);
  }
}
