import 'package:flutter/material.dart';

import 'camera_params.dart';
import 'swing_camera_controller.dart';

class SwingCameraView extends StatelessWidget {
  const SwingCameraView({
    Key? key,
    required this.controller,
  }) : super(key: key);

  final SwingCameraController controller;

  @override
  Widget build(BuildContext context) {
    return ValueListenableBuilder(
        valueListenable: controller.cameraParams,
        builder:
            (context, CameraParams? cameraParams, child) {
          if (cameraParams == null) {
            return const SizedBox();
          }

          return ClipRect(
            child: Transform.scale(
              scale: cameraParams.previewSize
                  .fill(MediaQuery.of(context).size),
              child: Center(
                child: AspectRatio(
                  aspectRatio:
                      cameraParams.previewSize.aspectRatio,
                  child: Texture(
                    textureId: int.tryParse(
                            cameraParams.cameraId) ??
                        0,
                  ),
                ),
              ),
            ),
          );
        });
  }
}

extension on Size {
  double fill(Size targetSize) {
    if (targetSize.aspectRatio < aspectRatio) {
      return targetSize.height *
          aspectRatio /
          targetSize.width;
    } else {
      return targetSize.width /
          aspectRatio /
          targetSize.height;
    }
  }
}
